==> mcai_worker_sdk-1.0.0-rc4.crate: mcai_worker_sdk-1.0.0-rc4/src/message_exchange/rabbitmq/publisher.rs <==
use super::{publish, CurrentOrders};
use crate::{
  message_exchange::{
    message::{Feedback, ResponseMessage},
    rabbitmq::current_orders::OrderDeliveryType,
  },
  MessageError, Result,
};
use async_std::{
  channel::{self, Receiver, Sender},
  sync::{Arc, Mutex as AsyncMutex},
  task::{self, JoinHandle},
};
use lapin::Channel;
use std::sync::Mutex;

pub struct RabbitmqPublisher {
  handle: Option<JoinHandle<()>>,
  response_sender: Sender<ResponseMessage>,
}

impl RabbitmqPublisher {
  pub async fn new(channel: &Channel, current_orders: Arc<Mutex<CurrentOrders>>) -> Result<Self> {
    let (response_sender, response_receiver) = channel::unbounded();

    let response_receiver = Arc::new(AsyncMutex::new(response_receiver));

    let channel = Arc::new(channel.clone());

    let handle = Some(task::spawn(async move {
      loop {
        if let Err(error) = Self::handle_response(
          response_receiver.clone(),
          channel.clone(),
          current_orders.clone(),
        )
        .await
        {
          log::error!("{:?}", error);
        }
      }
    }));

    Ok(RabbitmqPublisher {
      handle,
      response_sender,
    })
  }

  pub async fn send_response(&self, response: ResponseMessage) {
    self.response_sender.send(response).await.unwrap();
  }

  async fn respond_with_delivery(
    channel: Arc<Channel>,
    current_orders: Arc<Mutex<CurrentOrders>>,
    order_type: OrderDeliveryType,
    response: ResponseMessage,
  ) {
    let unacknowledged_delivery = current_orders
      .lock()
      .unwrap()
      .get_order_delivery_for(order_type.clone())
      .unwrap()
      .get_unacknowledged_delivery();
    match publish::response_with_delivery(
      channel.clone(),
      unacknowledged_delivery.clone(),
      &response,
    )
    .await
    {
      Ok(_) => {
        if unacknowledged_delivery.is_some() {
          current_orders
            .lock()
            .unwrap()
            .get_order_delivery_for(order_type.clone())
            .unwrap()
            .set_acknowledged()
        }
      }
      Err(error) => {
        if let Err(error) = publish::error(channel.clone(), unacknowledged_delivery, &error).await {
          log::error!("Unable to publish response: {:?}", error);
        }
      }
    }
  }

  async fn handle_response(
    response_receiver: Arc<AsyncMutex<Receiver<ResponseMessage>>>,
    channel: Arc<Channel>,
    current_orders: Arc<Mutex<CurrentOrders>>,
  ) -> Result<()> {
    let response = response_receiver.lock().await.recv().await.map_err(|e| {
      MessageError::RuntimeError(format!(
        "unable to wait response from processor: {:?}",
        e.to_string()
      ))
    })?;

    log::debug!("Response: {:?}", response);
    log::debug!("{}", current_orders.lock().unwrap());

    match &response {
      ResponseMessage::Feedback(Feedback::Progression(progression)) => {
        publish::job_progression(channel, progression.clone())?;
      }
      ResponseMessage::WorkerCreated(_) => {
        publish::response_with_delivery(channel.clone(), None, &response.clone()).await?;
      }
      ResponseMessage::WorkerTerminated(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Kill)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Kill,
            response.clone(),
          )
          .await;
        } else {
          publish::response_with_delivery(channel.clone(), None, &response.clone()).await?;
        }
      }
      ResponseMessage::WorkerInitialized(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Init)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Init,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Job)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Job,
            response.clone(),
          )
          .await;
        }
      }
      ResponseMessage::WorkerStarted(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Start)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Start,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Job)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Job,
            response.clone(),
          )
          .await;
        }
      }
      ResponseMessage::JobStopped(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Stop)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Stop,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Job)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Job,
            response.clone(),
          )
          .await;
        }
      }
      ResponseMessage::Completed(_) | ResponseMessage::Error(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Job)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Job,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Stop)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Stop,
            response.clone(),
          )
          .await;
        }
      }
      ResponseMessage::Feedback(_) | ResponseMessage::StatusError(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Status)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Status,
            response.clone(),
          )
          .await;
        }
      }
    }

    match response {
      ResponseMessage::WorkerCreated(_)
      | ResponseMessage::WorkerInitialized(_)
      | ResponseMessage::WorkerStarted(_)
      | ResponseMessage::WorkerTerminated(_) => {}
      ResponseMessage::Completed(_)
      | ResponseMessage::Error(_)
      | ResponseMessage::JobStopped(_) => {
        current_orders.lock().unwrap().reset_process_deliveries();
      }
      ResponseMessage::Feedback(_) | ResponseMessage::StatusError(_) => {
        current_orders.lock().unwrap().reset_status_deliveries();
      }
    };

    Ok(())
  }
}

impl Drop for RabbitmqPublisher {
  fn drop(&mut self) {
    self.handle.take().map(JoinHandle::cancel);
  }
}

==> mcai_worker_sdk-1.1.0-rc2.crate: mcai_worker_sdk-1.1.0-rc2/src/message_exchange/rabbitmq/publisher.rs <==
use super::{publish, CurrentOrders};
use crate::{
  message_exchange::{
    message::{Feedback, ResponseMessage},
    rabbitmq::current_orders::OrderDeliveryType,
  },
  MessageError, Result,
};
use async_std::{
  channel::{self, Receiver, Sender},
  sync::{Arc, Mutex as AsyncMutex},
  task::{self, JoinHandle},
};
use lapin::Channel;
use std::sync::Mutex;

pub struct RabbitmqPublisher {
  handle: Option<JoinHandle<()>>,
  response_sender: Sender<ResponseMessage>,
}

impl RabbitmqPublisher {
  pub async fn new(channel: &Channel, current_orders: Arc<Mutex<CurrentOrders>>) -> Result<Self> {
    let (response_sender, response_receiver) = channel::unbounded();

    let response_receiver = Arc::new(AsyncMutex::new(response_receiver));

    let channel = Arc::new(channel.clone());

    let handle = Some(task::spawn(async move {
      loop {
        if let Err(error) = Self::handle_response(
          response_receiver.clone(),
          channel.clone(),
          current_orders.clone(),
        )
        .await
        {
          log::error!("{:?}", error);
        }
      }
    }));

    Ok(RabbitmqPublisher {
      handle,
      response_sender,
    })
  }

  pub async fn send_response(&self, response: ResponseMessage) {
    self.response_sender.send(response).await.unwrap();
  }

  async fn respond_with_delivery(
    channel: Arc<Channel>,
    current_orders: Arc<Mutex<CurrentOrders>>,
    order_type: OrderDeliveryType,
    response: ResponseMessage,
  ) {
    let unacknowledged_delivery = current_orders
      .lock()
      .unwrap()
      .get_order_delivery_for(order_type.clone())
      .unwrap()
      .get_unacknowledged_delivery();
    match publish::response_with_delivery(
      channel.clone(),
      unacknowledged_delivery.clone(),
      &response,
    )
    .await
    {
      Ok(_) => {
        if unacknowledged_delivery.is_some() {
          current_orders
            .lock()
            .unwrap()
            .get_order_delivery_for(order_type.clone())
            .unwrap()
            .set_acknowledged()
        }
      }
      Err(error) => {
        if let Err(error) = publish::error(channel.clone(), unacknowledged_delivery, &error).await {
          log::error!("Unable to publish response: {:?}", error);
        }
      }
    }
  }

  async fn handle_response(
    response_receiver: Arc<AsyncMutex<Receiver<ResponseMessage>>>,
    channel: Arc<Channel>,
    current_orders: Arc<Mutex<CurrentOrders>>,
  ) -> Result<()> {
    let response = response_receiver.lock().await.recv().await.map_err(|e| {
      MessageError::RuntimeError(format!(
        "unable to wait response from processor: {:?}",
        e.to_string()
      ))
    })?;

    log::debug!("Response: {:?}", response);
    log::debug!("{}", current_orders.lock().unwrap());

    match &response {
      ResponseMessage::Feedback(Feedback::Progression(progression)) => {
        publish::job_progression(channel, progression.clone())?;
      }
      ResponseMessage::WorkerCreated(_) => {
        publish::response_with_delivery(channel.clone(), None, &response.clone()).await?;
      }
      ResponseMessage::WorkerTerminated(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Kill)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Kill,
            response.clone(),
          )
          .await;
        } else {
          publish::response_with_delivery(channel.clone(), None, &response.clone()).await?;
        }
      }
      ResponseMessage::WorkerInitialized(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Init)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Init,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Job)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Job,
            response.clone(),
          )
          .await;
        }
      }
      ResponseMessage::WorkerStarted(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Start)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Start,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Job)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Job,
            response.clone(),
          )
          .await;
        }
      }
      ResponseMessage::WorkerUpdated(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Update)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Update,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Job)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Job,
            response.clone(),
          )
          .await;
        }
      }
      ResponseMessage::JobStopped(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Stop)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Stop,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Job)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Job,
            response.clone(),
          )
          .await;
        }
      }
      ResponseMessage::Completed(_) | ResponseMessage::Error(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Job)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Job,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Stop)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Stop,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::ReachedExpiration)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::ReachedExpiration,
            response.clone(),
          )
          .await;
        }
      }
      ResponseMessage::Feedback(_) | ResponseMessage::StatusError(_) => {
        if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Suspend)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Suspend,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Resume)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Resume,
            response.clone(),
          )
          .await;
        } else if current_orders
          .lock()
          .unwrap()
          .has_order_delivery_for(OrderDeliveryType::Status)
        {
          Self::respond_with_delivery(
            channel,
            current_orders.clone(),
            OrderDeliveryType::Status,
            response.clone(),
          )
          .await;
        }
      }
    }

    match response {
      ResponseMessage::WorkerCreated(_)
      | ResponseMessage::WorkerInitialized(_)
      | ResponseMessage::WorkerStarted(_)
      | ResponseMessage::WorkerUpdated(_)
      | ResponseMessage::WorkerTerminated(_) => {}
      ResponseMessage::Completed(_)
      | ResponseMessage::Error(_)
      | ResponseMessage::JobStopped(_) => {
        current_orders.lock().unwrap().reset_process_deliveries();
      }
      ResponseMessage::Feedback(_) | ResponseMessage::StatusError(_) => {
        current_orders.lock().unwrap().reset_status_deliveries();
      }
    };

    Ok(())
  }
}

impl Drop for RabbitmqPublisher {
  fn drop(&mut self) {
    self.handle.take().map(JoinHandle::cancel);
  }
}
